#
#       Copyright (C) 2014
#       Sean Poyser (seanpoyser@gmail.com) and Richard Dean (write2dixie@gmail.com)
#
#  This Program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2, or (at your option)
#  any later version.
#
#  This Program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with XBMC; see the file COPYING.  If not, write to
#  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
#  http://www.gnu.org/copyleft/gpl.html
#

import os
import vpn_utils as utils
import path
import subprocess

#-------------------------------------------------------------


def install(silent=False):
    if utils.ADDON.getSetting('OS') == 'Windows':
        return

    if utils.ADDON.getSetting('OS') == 'MacOS':
        installMacOS()
        return

    # if 'OpenELEC' in utils.ADDON.getSetting('OS'):
    #     installOpenELEC()
    #     return

    if utils.ADDON.getSetting('OS') == 'EmbER':
        installEmber()
        return

    if utils.ADDON.getSetting('OS') == 'Android':
        installAndroid()
        return

    cmdLine  = utils.getSudo()
    cmdLine +='apt-get update;'
    cmdLine +='sudo apt-get -y install openvpn;'
    cmdLine +='sudo apt-get -y install psmisc'

    dp = None
    if not silent:
        dp = utils.progress('Installing OpenVPN application', 'Please be patient this may take a few minutes')

    p = subprocess.Popen(cmdLine, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)

    (stdout, stderr) = p.communicate()

    if silent:
        return

    import xbmc
    xbmc.sleep(100)
    dp.close()

    success = path.getPath(utils.ADDON.getSetting('OS'), silent=True)

    if success:
        utils.dialogOK('OpenVPN application successfully installed')
    else:
        utils.dialogOK('OpenVPN application installation failed', 'Please try again later')


def installMacOS():
    utils.log('============ Install MacOS ============')
    import download
    import extract
    import stat

    url    = utils.getResourceUrl() + 'openvpn-macos-2.3.4.zip'
    bindir = xbmc.translatePath('special://profile/addon_data/plugin.program.vpnicity/macos/sbin/')
    dest   = os.path.join(bindir, 'openvpn-macos.zip')
    macbin = os.path.join(bindir, 'openvpn')

    try:
        os.makedirs(bindir)
    except:
        pass

    download.download(url, dest)
    extract.all(dest, bindir)

    st = os.stat(macbin)
    os.chmod(macbin, st.st_mode | stat.S_IEXEC)

    try:
        os.remove(dest)
    except:
        pass

    success = path.getPath(utils.ADDON.getSetting('OS'), silent=True)

    if success:
        utils.dialogOK('MacOS OpenVPN application successfully installed')
    else:
        utils.dialogOK('MacOS OpenVPN application installation failed', 'Please try again later')


def installEmber():
    import download
    import extract
    import stat

    url      = utils.getResourceUrl() + 'openvpn-ember.zip'
    bindir   = xbmc.translatePath('special://profile/addon_data/plugin.program.vpnicity/ember/')
    dest     = os.path.join(bindir, 'openvpn-ember.zip')
    emberbin = os.path.join(bindir, 'openvpn')

    try:
        os.makedirs(bindir)
    except:
        pass

    download.download(url, dest)
    extract.all(dest, bindir)

    st = os.stat(emberbin)
    os.chmod(emberbin, st.st_mode | stat.S_IEXEC)

    try:
        os.remove(dest)
    except:
        pass

    success = path.getPath(utils.ADDON.getSetting('OS'), silent=True)

    if success:
        utils.dialogOK('Ember OpenVPN application successfully installed')
    else:
        utils.dialogOK('Ember OpenVPN application installation failed', 'Please try again later')


def installAndroid():
    import sfile
    import download

    vurl = 'http://plai.de/android/ics-openvpn-latest-stable.apk'
    home =  utils.RESOURCES
    curl =  os.path.join(home, 'VPNicity OpenVPN Control v1.0.0.apk')
    dst  =  xbmc.translatePath('special://profile/addon_data/plugin.program.vpnicity/android/')
    vpn  =  os.path.join(dst, 'ics-openvpn-latest-stable.apk')
    ctrl =  os.path.join(dst, 'VPNicity OpenVPN Control v1.0.0.apk')

    try:
        os.makedirs(dst)
    except:
        pass

    utils.dialogOK('We will now download and install [COLOR orange]VPNicity for Android[/COLOR].', 'Click the [COLOR orange]INSTALL[/COLOR] and [COLOR orange]DONE[/COLOR] buttons that show on screen')
    sfile.copy(curl, ctrl)
    xbmc.executebuiltin('StartAndroidActivity("","android.intent.action.VIEW","application/vnd.android.package-archive","file:'+ ctrl +'")')
    xbmc.sleep(3000)

    utils.dialogOK('We will now download and install [COLOR orange]OpenVPN for Android[/COLOR].', 'Click the [COLOR orange]INSTALL[/COLOR] + [COLOR orange]DONE[/COLOR] buttons that show on screen')
    download.download(vurl, vpn)
    xbmc.executebuiltin('StartAndroidActivity("","android.intent.action.VIEW","application/vnd.android.package-archive","file:'+ vpn +'")')
    xbmc.sleep(3000)


if __name__ == '__main__':
    install()
    # utils.ADDON.openSettings()